clear all, close all
%% setting

Fs = 1000; % [Hz]
dt = 1/Fs; % [s]
t_tot = 100; %[s]
t = 0:1/Fs:t_tot; %[s]

f_c = 45; % [Hz]
f_t = 65; % [Hz]
eps_ct = 12; % [Hz]
eps_tc = 4*eps_ct; % [Hz]
f0 = (eps_tc*f_c + eps_ct*f_t)/(eps_tc + eps_ct);

T_eff_set = 0.5; % [rad/s] * 2pi
sigma = sqrt(Fs*2*T_eff_set*2*pi);


%% show interflagellar synchrony
eps_f = 0;
f_f = 55;

[phi_c, phi_t, phi_f] = deal(zeros(size(t)));

x_c = rand(size(t));
zeta_c    = norminv(x_c,0,sigma/sqrt(2));
x_t = rand(size(t));
zeta_t    = norminv(x_t,0,sigma/sqrt(2));


phi_c(1) = rand()*2*pi;
phi_t(1) = rand()*2*pi;
phi_f(1) = rand()*2*pi;
for i = 1:length(t)-1
    d_phi_f = 2*pi*f_f*dt;
    phi_f(i+1) = phi_f(i)+ d_phi_f;

    d_phi_c = (2*pi*f_c - 2*pi*eps_ct * sin(phi_c(i)-phi_t(i))...
               - 2*pi*eps_f * sin(phi_c(i)-phi_f(i))+zeta_c(i))*dt;
    phi_c(i+1) =  phi_c(i)+ d_phi_c;

    d_phi_t = (2*pi*f_t - 2*pi*eps_tc * sin(phi_t(i)-phi_c(i))...
               - 2*pi*eps_f * sin(phi_t(i)-phi_f(i))+zeta_t(i))*dt;
    phi_t(i+1) =  phi_t(i)+ d_phi_t;
end

%% interflagellar phase difference

figure(1)
set(gcf,'DefaultAxesFontSize',10,'DefaultAxesFontWeight','normal',...
    'DefaultAxesLineWidth',1.5,'unit','normalized',...
    'position',[0.2,0.2,0.40,0.15],'PaperPositionMode','auto',...
    'DefaultTextInterpreter','Latex')
set(gca,'DefaultTextInterpreter','Latex',...
    'TickLabelInterpreter','latex','FontSize',10)
hold on, grid on, box on
xlabel('Time (s)')
ylabel('Flagellar phase (2$\pi$)')
idx_after_1s = [1000:length(t)];
t = t(idx_after_1s);
t = t-t(1);
phi_c = phi_c(idx_after_1s);
phi_t = phi_t(idx_after_1s);
phi_c = phi_c - phi_c(1);
phi_t = phi_t - phi_t(1);

plot(t, (phi_c - phi_t)/2/pi, 'k-', 'LineWidth',1)
xlim([0,11])
xticks([0:2:12])
ylim([-0.7,0.7])



%% flow-flag synchronization
eps_f = 2.4;
f_f_list = [44,46,47,49];

figure(2)
set(gcf,'DefaultAxesFontSize',10,'DefaultAxesFontWeight','normal',...
    'DefaultAxesLineWidth',1.5,'unit','normalized',...
    'position',[0.2,0.2,0.40,0.45],'PaperPositionMode','auto',...
    'DefaultTextInterpreter','Latex')
set(gca,'DefaultTextInterpreter','Latex',...
    'TickLabelInterpreter','latex')
hold on, grid on, box on


for i_f = 1:4
    f_f = f_f_list(i_f);
    
    [phi_c, phi_t, phi_f] = deal(zeros(size(t)));

    x_c = rand(size(t));
    zeta_c    = norminv(x_c,0,sigma/sqrt(2));
    x_t = rand(size(t));
    zeta_t    = norminv(x_t,0,sigma/sqrt(2));

    phi_c(1) = rand()*2*pi;
    phi_t(1) = rand()*2*pi;
    phi_f(1) = rand()*2*pi;
    for i = 1:length(t)-1
        d_phi_f = 2*pi*f_f*dt;
        phi_f(i+1) = phi_f(i)+ d_phi_f;

        d_phi_c = (2*pi*f_c - 2*pi*eps_ct * sin(phi_c(i)-phi_t(i))...
                   - 2*pi*eps_f * sin(phi_c(i)-phi_f(i))+zeta_c(i))*dt;
        phi_c(i+1) =  phi_c(i)+ d_phi_c;

        d_phi_t = (2*pi*f_t - 2*pi*eps_tc * sin(phi_t(i)-phi_c(i))...
                   - 2*pi*eps_f * sin(phi_t(i)-phi_f(i))+zeta_t(i))*dt;
        phi_t(i+1) =  phi_t(i)+ d_phi_t;
    end
    
    idx_after_1s = [1000:length(t)];
    t = t(idx_after_1s);
    t = t-t(1);
    phi_c = phi_c(idx_after_1s);
    phi_f = phi_f(idx_after_1s);
    phi_c = phi_c - phi_c(1);
    phi_f = phi_f - phi_f(1);
    dphi = (phi_c - phi_f)/2/pi;
    plot(t, dphi, 'k-', 'LineWidth',1.5, 'color',[0.6,0.6,0.6])
    idx_sync = find(diff(smooth(dphi,200))<1e-3);
    plot(t(idx_sync),dphi(idx_sync),'.','MarkerSize',9,'Color','k')
    disp(length(idx_sync)/length(t))
end

xlim([0,11])
xticks([0:2:12])
ylim([-0.7,20])
yticks([0:4:20])

%% run
eps_f_list =  2.4;
f_f_list = 42:56;
N_eps = length(eps_f_list);
N_freq = length(f_f_list);
[xi_ct_mat, xi_cf_mat, xi_tf_mat] = deal(zeros(N_eps,N_freq));

for i_eps = 1:N_eps
    fprintf('epsilon: %d/%d\n',i_eps,N_eps)
    for i_f = 1:N_freq
        eps_f = eps_f_list(i_eps);
        f_f = f_f_list(i_f);
                
        [phi_c, phi_t, phi_f] = deal(zeros(size(t)));

        x_c = rand(size(t));
        zeta_c    = norminv(x_c,0,sigma/sqrt(2));
        x_t = rand(size(t));
        zeta_t    = norminv(x_t,0,sigma/sqrt(2));

        
        phi_c(1) = rand()*2*pi;
        phi_t(1) = rand()*2*pi;
        phi_f(1) = rand()*2*pi;
        for i = 1:length(t)-1
            d_phi_f = 2*pi*f_f*dt;
            phi_f(i+1) = phi_f(i)+ d_phi_f;
            
            d_phi_c = (2*pi*f_c - 2*pi*eps_ct * sin(phi_c(i)-phi_t(i))...
                       - 2*pi*eps_f * sin(phi_c(i)-phi_f(i))+zeta_c(i))*dt;
            phi_c(i+1) =  phi_c(i)+ d_phi_c;
            
            d_phi_t = (2*pi*f_t - 2*pi*eps_tc * sin(phi_t(i)-phi_c(i))...
                       - 2*pi*eps_f * sin(phi_t(i)-phi_f(i))+zeta_t(i))*dt;
            phi_t(i+1) =  phi_t(i)+ d_phi_t;
        end
        
        Delta_ct = phi_c - phi_t;
        Delta_cf = phi_c - phi_f;
        Delta_tf = phi_t - phi_f;
        [~,xi_ct_mat(i_eps,i_f),~] = markSync(Delta_ct/2/pi,Fs);
        [~,xi_cf_mat(i_eps,i_f),~] = markSync(Delta_cf/2/pi,Fs);
        [~,xi_tf_mat(i_eps,i_f),~] = markSync(Delta_tf/2/pi,Fs);
    end
end

figure(3)
plot(f_f_list - f0, xi_cf_mat, 'o-')
ylabel('\tau')
xlabel('\nu (Hz)')
