%{
This generate the modelled Sync. profiles.
Asymmetry in interflagellar coupling is tuned by "cis_alphas", using the
    notations in the main text:
            cis_alpha = lambda_cis  / (lambda_cis+lambda_trans)
    
For wildtype in TRIS:
    T_eff_set = 0.5 [*2pi]; (Note T_eff^{c,t}=T_eff_set/2)
    cis_alpha = 1/5;        (lambda_cis = 4lambda_trans)
    eps_f     = 2.4 Hz      (experimentally measured for axial flow)
For wildtype in EGTA：
    T_eff_set = 0.5 [*2pi];
    cis_alpha = 1/5; 
    eps_f = 2.04 Hz         (to reflect the shrinkage in sync. profiles)
For ptx1 in TRIS:
    T_eff_set = 3.0 [*2pi]
    cis_alpha = 1/5;        (lambda_cis = lambda_trans)
    eps_f = 2.4 Hz
%}

%%
close all

Fs = 1000; % [Hz]
dt = 1/Fs; % [s]
t_tot = 100; %[s]
t = 0:1/Fs:t_tot; %[s]

f_c = 45; % [Hz]
f_t = 65; % [Hz]

lambda_tot = 60;

color_list = {'r','g','b'};
coef_ct_list = {[2,1],[1.5, 1.5],[1,2]};
label_list = {'cis flow', 'axial flow', 'trans flow'};

cis_alphas = [2/3, 1/2, 1/3, 1/5, 1/7, 1/9, 1/11, 0];
exp_label_list = {'(1:2)', '(1:1)', '(2:1)', '(4:1)',...
              '(6:1)', '(8:1)', '(10:1)', '(10:0)'};


          
for iiii = 1:numel(cis_alphas)
    cis_alpha = cis_alphas(iiii);
    exp_label = exp_label_list{iiii};

    lambda_ct = lambda_tot * cis_alpha ; % [Hz]
    lambda_tc = lambda_tot - lambda_ct; % [Hz]
    T_eff_set = 0.5; % [/s] * 2pi  %0.5 for wt; 3.0 for ptx1
    sigma = sqrt(Fs*2*T_eff_set*2*pi);
    f0 = (lambda_tc*f_c + lambda_ct*f_t)/(lambda_tc + lambda_ct);
    fprintf('f0 = %.2f Hz\n',f0)
    
    eps_f =  2.4/1.5;  % make sure axial flow eps = 2.4/1.5
                       % for calcium depletion use eps_f = 2.04/1.5
    f_f_list = [-6:0.3:6]+f0;
    N_freq = length(f_f_list);
    [xi_cf_mat, xi_tf_mat] = deal(zeros(3,N_freq));
    h_list = cell(3,1);
    
    
    for i_flow = 1:3
        coef_ct = coef_ct_list{i_flow};
        coef_c = coef_ct(1);
        coef_t = coef_ct(2);

        for i_f = 1:N_freq
            f_f = f_f_list(i_f);

            [phi_c, phi_t, phi_f] = deal(zeros(size(t)));

            x_c = rand(size(t));
            zeta_c    = norminv(x_c,0,sigma/sqrt(2));
            x_t = rand(size(t));
            zeta_t    = norminv(x_t,0,sigma/sqrt(2));


            phi_c(1) = rand()*2*pi;
            phi_t(1) = rand()*2*pi;
            phi_f(1) = rand()*2*pi;
            for i = 1:length(t)-1
                d_phi_f = 2*pi*f_f*dt;
                phi_f(i+1) = phi_f(i)+ d_phi_f;

                d_phi_c = (2*pi*f_c - 2*pi*lambda_ct * sin(phi_c(i)-phi_t(i))...
                           - 2*pi*eps_f*coef_c * sin(phi_c(i)-phi_f(i))+zeta_c(i))*dt;
                phi_c(i+1) =  phi_c(i)+ d_phi_c;

                d_phi_t = (2*pi*f_t - 2*pi*lambda_tc * sin(phi_t(i)-phi_c(i))...
                           - 2*pi*eps_f*coef_t * sin(phi_t(i)-phi_f(i))+zeta_t(i))*dt;
                phi_t(i+1) =  phi_t(i)+ d_phi_t;
            end

            Delta_ct = phi_c - phi_t;
            Delta_cf = (phi_c - phi_f);
            Delta_tf = (phi_t - phi_f);
            [~,xi_cf_mat(i_flow,i_f),~] = markSync(Delta_cf/2/pi,Fs);
            [~,xi_tf_mat(i_flow,i_f),~] = markSync(Delta_tf/2/pi,Fs);
        end
    end

    %% plot simulation
    figure()
    set(gcf,'Units','inches','Position',[1,4,4,2],...
        'defaultTextInterpreter','Latex','defaultAxesFontSize',10)
    set(gca,'defaultTextInterpreter','Latex',...
        'tickLabelInterpreter','Latex')
    hold on , box on, grid on
    title('Simulation')
    xlabel('$f_{flow}-f_0$ (Hz)')
    ylabel('$\xi(\nu)$ (Hz)')
    xlim([-7,7]),ylim([-0.05,1.05])
    for i_flow = 1:3

        color = color_list{i_flow};
        h_list{i_flow} = plot(f_f_list-f0, xi_cf_mat(i_flow,:),...
                              'LineWidth', 1.5, 'Color', color);
    end
    legend([h_list{:}],label_list,'Location', 'northeast',...
        'Interpreter','latex')
    title(exp_label,'FontSize',12)
end